package es.us.eii.pfc.jcrbib.utils;

import static es.us.eii.pfc.jcrbib.utils.ConstantsUtils.LOGGERNAME;
import static es.us.eii.pfc.jcrbib.utils.ConstantsUtils.PATHSEP;
import static es.us.eii.pfc.jcrbib.utils.ConstantsUtils.REPOSITORYINITRELPATH;
import static es.us.eii.pfc.jcrbib.utils.ConstantsUtils.REPOSITORYNAME;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.Hashtable;

import javax.jcr.ImportUUIDBehavior;
import javax.jcr.Node;
import javax.jcr.PathNotFoundException;
import javax.jcr.Repository;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.ServletContext;

import org.apache.jackrabbit.core.jndi.RegistryHelper;
import org.apache.log4j.Logger;

/**
 * 
 * Clase para la interaccin con el repositorio
 * 
 * 
 * 
 */
public class RepositoryUtils {

	/**
	 * Ruta del fichero de configuracion repository.xml
	 */
	private static String configFile = "webapps" + PATHSEP + "jcrbib" + PATHSEP
			+ "WEB-INF" + PATHSEP + "repository.xml";

	private static String repHomeDir = REPOSITORYNAME;

	/**
	 * Atributo que almacena la instancia del repositorio
	 */
	private static Repository repositorySingleton;

	private static Logger log;
	static {
		log = Logger.getLogger(LOGGERNAME);
		log.info("Log activado");
	}

	/**
	 * Mtodo que obtiene una instancia de repositorio
	 * 
	 * @return Una instancia de repositorio
	 * @throws IOException
	 *             Si no se encuentra el fichero de configuracin
	 */
	public static Repository getRepository() throws IOException {
		// Repository repo = null;
		log.debug("Obteniendo repositorio...");
		if (repositorySingleton == null) {

			Hashtable<String, String> env = new Hashtable<String, String>();
			env
					.put(Context.INITIAL_CONTEXT_FACTORY,
							"org.apache.jackrabbit.core.jndi.provider.DummyInitialContextFactory");

			env.put(Context.PROVIDER_URL, "localhost");

			InitialContext ctx;
			try {
				ctx = new InitialContext(env);
				RegistryHelper.registerRepository(ctx, "repo", configFile,
						repHomeDir, true);

				log.debug("Creando repositorio por primera vez");
				repositorySingleton = (Repository) ctx.lookup("repo");
			} catch (NamingException e) {
				log.error(e);
			} catch (RepositoryException e) {
				log.error("El repositorio ya estaba bloqueado");
				log.error(e);
			}
		}
		log.debug("Obtenido el repositorio con xito!");

		return repositorySingleton;
	}

	/**
	 * Mtodo que para la instancia de repositorio
	 */
	public static void shutdownRepository() {
		if (repositorySingleton != null) {
			log.debug("Empezando a desregistrar...");
			Hashtable<String, String> env = new Hashtable<String, String>();
			env
					.put(Context.INITIAL_CONTEXT_FACTORY,
							"org.apache.jackrabbit.core.jndi.provider.DummyInitialContextFactory");
			env.put(Context.PROVIDER_URL, "localhost");

			InitialContext ctx;
			try {
				ctx = new InitialContext(env);

				RegistryHelper.unregisterRepository(ctx, "repo");

				log.debug("Desregistrando el repositorio");
			} catch (NamingException e) {
				log.error(e);
			}
		}
	}

	/**
	 * 
	 * Mtodo que inicializa el repositorio
	 * 
	 * @param context
	 *            Contexto para calcular las rutas de ficheros
	 * @throws RepositoryException
	 *             Si hay algn problema durante la inicializacin
	 * @throws IOException
	 *             Si no se encuentra el fichero de inicializacin
	 */
	public static void initRepository(ServletContext context)
			throws RepositoryException, IOException {
		log.debug("Obteniendo el initPath...");
		String initPath = context.getRealPath(REPOSITORYINITRELPATH);
		log.debug("initPath: " + initPath);
		FileInputStream initXml = new FileInputStream(initPath);
		log.debug("Importando la estructura del repositorio...");
		Session repSession = SessionUtils.getSession();
		repSession.importXML("/", initXml,
				ImportUUIDBehavior.IMPORT_UUID_COLLISION_THROW);
		repSession.save();
		repSession.logout();
		log.debug("Estructura importada con xito!");
	}

	/**
	 * Mtodo que evala si el repositorio ha sido inicializado
	 * 
	 * @return true si el repositorio ha sido inicializado
	 * @throws RepositoryException
	 *             Si hay algn problema
	 */
	public static boolean hasBeenInit() throws RepositoryException {
		Session repSession = SessionUtils.getSession();
		Node root = repSession.getRootNode();
		boolean hasBeenInit = (root != null) && root.hasNode(REPOSITORYNAME);
		log.debug("El repositorio ha sido inicializado ya? " + hasBeenInit);
		repSession.logout();
		return hasBeenInit;
	}

	/**
	 * Mtodo que obtiene el nodo inicial de la biblioteca
	 * 
	 * @param s
	 *            Sesin a usar
	 * @return El nodo inicial
	 * @throws PathNotFoundException
	 *             Si no existe el nodo (el repositorio no est inicializado)
	 * @throws RepositoryException
	 *             Si hay algn problema
	 */
	public static Node getInitNode(Session s) throws PathNotFoundException,
			RepositoryException {
		return s.getRootNode().getNode(REPOSITORYNAME);
	}

	/**
	 * 
	 * Mtodo que vaca la informacin el repositorio
	 * 
	 * @param s
	 *            Sesin a utilizar
	 * @throws PathNotFoundException
	 *             Si el repositorio no ha sido inicializado
	 * @throws RepositoryException
	 *             Si hay algn problema
	 */
	public static void eraseRepository(Session s) throws PathNotFoundException,
			RepositoryException {
		Node init = getInitNode(s);
		log.debug("Borrando el repositorio...");
		if (hasBeenInit()) {
			init.remove();
			s.save();
			log.debug("Borrado!");
		}
	}
}
